#!/usr/bin/perl

##########################################################################
#
# This is the code implementing the Bastille-1.1.1 curses interface.
#
# In Bastille-1.1.1, this code was in the InteractiveBastille.pl script
# and carried the following copyright:
#
# Copyright (C) 2000 Jay Beale
# Licensed under the GNU General Public License
#
# The do_Bastille function is a wrapper around the main program logic
# from the InteractiveBastille.pl script.  When it is called, the
# Curses interface runs.  Other than turning this code into a perl
# module and adding the do_Bastille wrapper, there have been no
# changes.

##########################################
## Bastille Linux Text User Interface   ##
##########################################

#
# TO DO:
#
#  1) Use the newly-release Curses::Widgets 1.1 to get better text windows
#  2) Finish documenting program
#  3) Rewrite &Ask subroutine: either use Curses::Forms or just make cleaner

# This use is needed to give the TUI/GUI access to the GLOBAL_ hashes.

use API;   

sub do_Bastille {

    # Number_Modules is the number of modules loaded in by Load_Questions
    $Number_Modules=0;

    # Load in the questions database
    $Number_Modules=&Load_Questions;

    # Check for an existing config file and populate the default answers from it
    &Populate_Answers;

    $next_question_index="Title_Screen";
    while ($next_question_index ne "RUN_SCRIPT") {
        $next_question_index=&Ask($next_question_index);    
    }

    # Output answers to the script and display

    &Output_Config_Files;

    # Run Bastille

#    &Run_Bastille_with_Config;


    # Display Credits

    open CREDITS,"/usr/share/Bastille/Credits";
    while (my $creditline=<CREDITS>) {
        $credits .= $creditline;
    }
    close CREDITS;
    &showEpilogue($credits);
}

sub Ask {
# sub Ask (index to Question{} record)

##############################################################################
#
# More comments coming here, from Design Doc...
#
# WEIRD CASES:
#
#  1 If a record has no short_exp explanation, none is shown.  This is bad?
#  2 If a record has no question, no question is asked, but the explanation
#    is still displayed.  If this is the case, the default_answer is still
#    entered into the configuration, if it exists.
#  3 If a question has no answer, it doesn't create any blank lines or such
#    in the output file, as it will be skipped in &Output_Config_Files.  For
#    this reason, &Prompt_Input, which is only called when the record contains
#    a user-answerable question, pads a space to the end of any 0-length input.
#    Not to worry: Output_Config_Files replaces said space with 0-length input
#    NOTE: we couldn't just only print the answer field when a real question
#          existed -- this would improperly handle case 2.
#    
###############################################################################

    #
    # Load question into local variables
    #
    my $index=$_[0];
    my $short_exp      =$Question{$index}{"short_exp"};
    my $long_exp       =$Question{$index}{"long_exp"};
    my $question       =$Question{$index}{"question"};
    my $toggle_confirm = $Question{$index}{"toggle_confirm"};
    my $toggle_yn      =$Question{$index}{"toggle_yn"};
    my $yes_epilogue   =$Question{$index}{"yes_epilogue"};
    my $no_epilogue    =$Question{$index}{"no_epilogue"};
    my $yes_child      =$Question{$index}{"yes_child"};
    my $no_child       =$Question{$index}{"no_child"};
    my $proper_parent  =$Question{$index}{"proper_parent"};
    my $answer         =$Question{$index}{"answer"};
    my $module         =$Question{$index}{"module"};

    
    #
    # Set up other local variables
    #

    # Index to next question -- returned by this routine -- loop until we
    # have an index
    my $return_index="";

    # command line entry value
    my $command_input = "";
    

    while (not $return_index) {
	# while a question change is not requested show the main menu
	while($command_input ne "n" && $command_input ne "b" && $command_input ne "e"){
	    $command_input = &showMenu($question,$answer,$short_exp,$module);

	    if($command_input eq "a"){
		$answer = &promptAnswer($question,$answer,$toggle_yn);
		$command_input = "";
	    }
	    elsif($command_input eq "l"){
		
		my $explanation = "";
		if ($long_exp) {
		    $explanation="Q: $question\n$long_exp\n\n$question";
		}
		elsif($short_exp) {
		    $explanation="$short_exp\n\n$question";
		}
		else {
		    $explanation="No explanation is provided with this question.\n";
		}
		&showExplanation($explanation);
		$command_input = "";
	    }
	    elsif($command_input eq "s") {
		my $explanation;
		if ($short_exp) {
		    $explanation="Q: $question\n$short_exp\n\n$question";
		}
		else {
		    $explanation="No Explanation was provided for this question.\n";
		}
		&showExplanation($explanation);
		$command_input = "";
	    }
	    elsif($command_input eq "e"){
		$command_input = &showExitWarning;
	    }
	    elsif($command_input eq "h"){
		&showHelp("menu");
		$command_input = "";
	    }
	}
	
	
	# Did the user give the Back command?
	if ($command_input eq "b") {
	    $return_index=$proper_parent;
	}

	# Did the user give the next command on an asked question?

	if  ($command_input eq "n") {

	    # If not a Y/N question, use the Yes Child and Epilogue
	    unless ($toggle_yn) {		
		$return_index=$yes_child;
		
		if ($yes_epilogue) {
		    &showEpilogue($yes_epilogue);
		}
	    }
	    # Otherwise, use the epilogue appropriate to the answer
	    else {
		if ($answer=~/^\s*Y/i) {
		    $return_index=$yes_child;

		    &showEpilogue($yes_epilogue);
		}
		elsif ($answer =~/^\s*N/i) {
		    $return_index=$no_child;
		    &showEpilogue($no_epilogue);
		}
	    }
	}

	# Now, if the user hit Prev/Next, store the answer so it's there
	# when we come back to it...
	if ($return_index) {
	    if ($answer) {
		$Question{$index}{"answer"}=$answer;
	    }
	}

	# Deal with text-screen questions: questions w/ no question and
        # explanation which still have an epilogue

	if ((not $short_exp) and (not $question)) {
	    $return_index=$yes_child;
	    &showEpilogue($yes_epilogue);
	}

    }
    print "At return INDEX in ASK\n";
    return $return_index;
}


sub showMenu($$$$){
    my $question = $_[0];
    my $answer = $_[1];
    my $short_exp = $_[2];
    my $module = $_[3];
 
    my $validCommand = 0;
    my $command_input = "";
 
    while(not $validCommand){
	if($question){
	    print "$module\n"; 
	    print "Type \"help\" for usage information.\n\n";
	    print "Question:\n $question\n";
	    print "Current Answer: \"$answer\"\n";
	    print "BASTILLE%> ";

	    $command_input = <STDIN>;
	    chomp($command_input);
	    $command_input = lc($command_input);

	    if($command_input =~ /^\s*a\s*$|^\s*answer\s*$/){
		$command_input = "a";
		$validCommand = 1;
	    }
	    elsif($command_input =~ /^\s*b\s*$|^\s*back\s*$/){
		$command_input = "b";
		$validCommand = 1;
	    }		
	    elsif($command_input =~ /^\s*e\s*$|^\s*exit\s*$/){
		$command_input = "e";
		$validCommand = 1;
	    }		
	    elsif($command_input =~ /^\s*h\s*$|^\s*help\s*$/){
		$command_input = "h";
		$validCommand = 1;
	    }		
	    elsif($command_input =~ /^\s*l\s*$|^\s*long\s*$/){
		$command_input = "l";
		$validCommand = 1;
	    }		
	    elsif($command_input =~ /^\s*n\s*$|^\s*next\s*$/){
		$command_input = "n";
		$validCommand = 1;
	    }		
	    elsif($command_input =~ /^\s*s\s*$|^\s*short\s*$/){
		$command_input = "s";
		$validCommand = 1;
	    }		
	    else {
		print "\"$command_input\" is not a valid entry.\n";
	    }
	}
	elsif($short_exp){
	    my $more = &getGlobal('BIN',"more");
	    my $echo = &getGlobal('BIN',"echo");
#	    `$echo "$short_exp" | $more `;
	    print $short_exp;
	    my $stop = <STDIN>;
	    $command_input = "n";
	    $validCommand = 1;
	}
	else {
	    $command_input = "n";
	    $validCommand = 1;
	}
    }
    print "At return input in Menu\n";

    return $command_input;
}

sub promptAnswer($$$){

    my $question = $_[0];
    my $answer = $_[1];
    my $yn_toggle = $_[2];
    
    my $current_exp = "";
    my $command_input = "";
    my $valid_input = 0;

    while(not $valid_input){
	if($yn_toggle){
	    if($answer =~ /^Y$/ ){
		$current_exp = "This is a Yes/No question:\n$question\n" . 
		    "Current Value is \"$answer\"\nANSWER: [Y]/N> ";
	    }
	    else{
		$current_exp = "This is a Yes/No question:\n\n$question\n\n" . 
		    "The bracketed value is the current default.\nANSWER: Y/[N]> ";
	    }
	    print "$current_exp";
	    $command_input = <STDIN>;
	    chomp($command_input);
	    $command_input = uc($command_input);
    
	    if($command_input =~ /^\s*Y\s*$|^\s*N\s*$/){
		$answer = $command_input;
		$valid_input = 1;
	    }
	    elsif($command_input =~ /^\s*$/){
		$answer = $answer;
		$valid_input = 1;
	    }
	    else{
		print "\"$command_input\" is not a valid answer.\n";
	    }
	    
	}	    
	else {
	    $current_exp = "This question requires machine specific information.\n\n" .
		"$question\n\n" .
		    "Please answer the question at the prompt.\n\n" .
			"The current answer is \"$answer\"\n" . 
			    "ANSWER%> ";
	    print "$current_exp";
	    $command_input = <STDIN>;
	    chomp($command_input);
	    $valid_input = 1;
	}
    }
    return $answer;
}


sub showExplanation($){
    my $explanation = $_[0];

    my $more = &getGlobal('BIN',"more");
    my $echo = &getGlobal('BIN',"echo");
    `$echo "$explanation" | $more `;
    return;
}

sub showEpilogue($){
    my $epilogue = $_[0];

    my $more = &getGlobal('BIN',"more");
    my $echo = &getGlobal('BIN',"echo");
    `$echo "$epilogue" | $more `;
    return;
}

sub showExitWarning{

    my $valid_input = 0;
    my $command_input = "";
    while(not $valid_input){
	print "Are you sure you want to exit.\n";
	print "All of your changes will be lost.\n";
	print "You must complete all questions before\n" . 
	    "you quit to complete the configuration.\n" . 
		"Exit and loose changes? Y/[N]> ";
	$command_input = <STDIN>;
	chomp($command_input);
	$command_input = uc($command_input);

	if($command_input =~ /N|^\s*$/){
	    $valid_input = 1;
	    return "";
	}
	elsif($command_input =~ /Y/){
	    $valid_input = 1;
	    exit(0);
	}
    }
    
}

sub showHelp($){
    my $location = $_[0];
    my $usageStr = "";
    if($location eq "menu"){
	$usageStr = "INPUT:\n\"a\" or \"answer\" to answer the question\n" .
	    "NAVIGATION:\n\"b\" or \"back\" to move to the previous question\n" .
		"\"n\" or \"next\" to move to the next question\n" .
		    "\"e\" or \"exit\" to exit the configuration without saving changes\n" .
			"HELP:\n\"l\" or \"long\" to get a long explanation of the question\n" .
			    "\"s\" or \"short\" to get a short explanation of the question\n" .
				"\"h\" or \"help\" to get UI usage help\n";
	
    }

    my $more = &getGlobal('BIN',"more");
    my $echo = &getGlobal('BIN',"echo");
    `$echo "$usageStr" | $more `;
    return;
    
}
1;





